package api

import (
    "encoding/json"
    "fmt"
    "io/ioutil"
    "os"
    "strings"
)

type XYZ struct {
    Name              string

    ARequired         string
    AOptional         string
    AOptionalDefault  string
    AOptionalComputed string
    AComputed         string

    Embedded          []*XYZEmbedded
}

type XYZEmbedded struct {
    BRequired         string
    BOptional         string
    BOptionalDefault  string
    BOptionalComputed string
    BComputed         string
}

func (c *ABCClient) CreateXYZ(name string, properties *XYZ) error {
    // create the resource in the infrastructure

    if properties.AOptionalComputed == "" {
        properties.AOptionalComputed = "computed"
    }
    if properties.AComputed == "" {
        properties.AComputed = "computed"
    }

    // if properties.Embedded == nil {                                          // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //     properties.Embedded = make([]*XYZEmbedded, 1, 1)                     // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //     properties.Embedded[0] = &XYZEmbedded{                               // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //         BRequired:         "computed",                                   // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //         BOptionalDefault:  "computed",                                   // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //         BOptionalComputed: "computed",                                   // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //         BComputed:         "computed",                                   // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //     }                                                                    // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    // } else {                                                                 // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
        if ( len(properties.Embedded) > 0 ) {
            for _, resource := range properties.Embedded {
                if resource.BOptionalComputed == "" {
                    resource.BOptionalComputed = "computed"
                }
                if resource.BComputed == "" {
                    resource.BComputed = "computed"
                }
            }
        }
    // }                                                                        // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr

    file := fmt.Sprintf("./%s.json", name)

    _, err := os.Stat(file)
    if err == nil {
        err = fmt.Errorf("CreateXYZ(%q): xyz already exists.", name)
        return err
    }

    content, err := json.MarshalIndent(properties, "", "    ")
    if err != nil {
        return err
    }

    err = ioutil.WriteFile(file, content, 0644)
    if err != nil {
        return err
    }

    return nil
}

func (c *ABCClient) ReadXYZ(name string) (xyz *XYZ, err error) {
    // read the data-source's information from the infrastructure

    file := fmt.Sprintf("./%s.json", name)

    content, err := ioutil.ReadFile(file)
    if err != nil {
        if strings.Contains(err.Error(), "The system cannot find the file specified.") {
            err = fmt.Errorf("ReadXYZ(%q): cannot find xyz.", name)
        }
        return nil, err
    }

    xyz = new(XYZ)
    err = json.Unmarshal(content, xyz)
    if err != nil {
        return nil, err
    }

    return xyz, nil
}

func (c *ABCClient) UpdateXYZ(name string, properties *XYZ) error {
    // update the resource in the infrastructure

    if properties.AOptionalComputed == "" {
        properties.AOptionalComputed = "computed"
    }
    if properties.AComputed == "" {
        properties.AComputed = "computed"
    }

    // if properties.Embedded == nil {                                          // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //     properties.Embedded = make([]*XYZEmbedded, 1, 1)                     // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //     properties.Embedded[0] = &XYZEmbedded{                               // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //         BRequired:         "computed",                                   // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //         BOptionalDefault:  "computed",                                   // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //         BOptionalComputed: "computed",                                   // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //         BComputed:         "computed",                                   // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    //     }                                                                    // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
    // } else {                                                                 // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr
        if ( len(properties.Embedded) > 0 ) {
            for _, resource := range properties.Embedded {
                if resource.BOptionalComputed == "" {
                    resource.BOptionalComputed = "computed"
                }
                if resource.BComputed == "" {
                    resource.BComputed = "computed"
                }
            }
        }
    // }                                                                        // uncomment when Embedded is Computed or Optional-Computed & NOT schema.SchemaConfigModeAttr

    file := fmt.Sprintf("./%s.json", name)

    content, err := json.MarshalIndent(properties, "", "    ")
    if err != nil {
        return err
    }

    err = ioutil.WriteFile(file, content, 0644)
    if err != nil {
        return err
    }

    return nil
}

func (c *ABCClient) DeleteXYZ(name string) error {
    // delete the resource from the infrastructure

    file := fmt.Sprintf("./%s.json", name)

    err := os.Remove(file)
    if err != nil {
        return err
    }

    return nil
}